"""Helper methods for Hijri conversion."""


def jdn_to_ordinal(jdn: int) -> int:
    """Convert Julian day number (JDN) to Gregorian date ordinal.

    Args:
        jdn: Julian day number (JDN).
    """

    return jdn - 1721425


def ordinal_to_jdn(ordinal: int) -> int:
    """Convert Gregorian date ordinal to Julian day number (JDN).

    Args:
        ordinal: Gregorian date ordinal.
    """

    return ordinal + 1721425


def jdn_to_rjd(jdn: int) -> int:
    """Return Reduced Julian Day (RJD) number from Julian day number (JDN).

    Args:
        jdn: Julian day number (JDN).
    """

    return jdn - 2400000


def rjd_to_jdn(rjd: int) -> int:
    """Return Julian day number (JDN) from Reduced Julian Day (RJD) number.

    Args:
        rjd: Reduced Julian Day (RJD) number.
    """

    return rjd + 2400000



"""Setup.py entry point for supporting editable installs

Configuration is handled by setuptools through setup.cfg
https://setuptools.readthedocs.io/en/latest/setuptools.html
"""

import re
from pathlib import Path

import setuptools

# Clean the readme file from badges and markers
this_directory = Path(__file__).parent
readme_content = (this_directory / "README.md").read_text()
readme_content_no_badges = re.sub(
    r"<!-- start badges -->[\S\s]+<!-- end badges -->\n+",
    "",
    readme_content,
)
readme_content_cleaned = re.sub(
    r"(<!-- .* -->)\n+", "", readme_content_no_badges
)

# Package name is added here for GitHub's dependency graph
# Also, a cleaner version of the long description is added
setuptools.setup(
    name="hijridate",
    long_description=readme_content_cleaned,
    long_description_content_type="text/markdown",
)



import pytest

from hijridate import locales


@pytest.fixture(scope="class")
def all_locales(request):
    request.cls.locales = locales._locale_map


@pytest.mark.usefixtures("all_locales")
class TestLocalesValidity:
    def test_locale_data_structure(self):
        for locale_cls in self.locales.values():
            assert len(locale_cls.language_tag) == 2
            assert locale_cls.language_tag.islower()
            assert len(locale_cls.month_names) == 12
            assert all(locale_cls.month_names)  # not blank or None
            assert len(locale_cls.gregorian_month_names) == 12
            assert all(locale_cls.gregorian_month_names)  # not blank or None
            assert len(locale_cls.day_names) == 7
            assert all(locale_cls.day_names)  # not blank or None
            assert locale_cls.notation is not None
            assert locale_cls.gregorian_notation is not None

    def test_locale_map(self):
        assert len(locales._locale_map) > 0
        assert "en" in locales._locale_map.keys()

    def test_duplicated_language_tag(self):
        with pytest.raises(LookupError):

            class ExtraLocale(locales.Locale):
                language_tag = "en"


class TestGettingLocale:
    class CustomLocale(locales.EnglishLocale):
        language_tag = "xx"

    @pytest.mark.parametrize(
        "test_input",
        [
            "xx",
            "XX",
            "xx-YY",
            "xx-yy",
            "xx_yy",
            "xx_YY",
            "xx_YY.UTF-8",
        ],
    )
    def test_locale_possible_names(self, test_input):
        assert locales.get_locale(test_input).__class__ == self.CustomLocale

    def test_unsupported_language(self):
        with pytest.raises(ValueError):
            locales.get_locale("xy")
